/**
 *
 *  Copyright Openmind http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sourceforge.openutils.mgnlcriteria.jcr.query.Criteria;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.JCRQueryException;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Criterion;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.criterion.Order;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * A generic Criteria implementation. Concrete subclasses must handle a QueryManager.
 * @author fgrilli
 * @version $Id: AbstractCriteriaImpl.java 1398 2009-09-05 18:28:47Z fgiust $
 */
public abstract class AbstractCriteriaImpl implements Criteria
{

    private static final long serialVersionUID = -66324747458323718L;

    protected static final Logger log = LoggerFactory.getLogger(AbstractMagnoliaCriteriaImpl.class);

    protected String path = Criterion.JCR_ROOT;

    protected String itemType;

    protected Class< ? > classType;

    private List<CriterionEntry> criterionEntries = new ArrayList<CriterionEntry>();

    private List<OrderEntry> orderEntries = new ArrayList<OrderEntry>();

    private Integer maxResults = Integer.MAX_VALUE;

    private Integer firstResult = 0;

    private Integer timeout;

    private String comment;

    protected AbstractCriteriaImpl()
    {
        super();
    }

    public Iterator<CriterionEntry> iterateExpressionEntries()
    {
        return criterionEntries.iterator();
    }

    public Iterator<OrderEntry> iterateOrderings()
    {
        return orderEntries.iterator();
    }

    public Criteria add(Criteria criteriaInst, Criterion expression)
    {
        criterionEntries.add(new CriterionEntry(expression, criteriaInst));
        return this;
    }

    public Criteria add(Criterion expression)
    {
        add(this, expression);
        return this;
    }

    public Criteria addOrder(Order ordering)
    {
        orderEntries.add(new OrderEntry(ordering, this));
        return this;
    }

    public Integer getMaxResults()
    {
        return maxResults;
    }

    public Criteria setMaxResults(int maxResults)
    {
        this.maxResults = maxResults;
        return this;
    }

    public Integer getFirstResult()
    {
        return firstResult;
    }

    public Criteria setFirstResult(int firstResult)
    {
        this.firstResult = firstResult;
        return this;
    }

    public Integer getTimeout()
    {
        return timeout;
    }

    public Criteria setTimeout(int timeout)
    {
        this.timeout = timeout;
        return this;
    }

    public String getComment()
    {
        return comment;
    }

    public Criteria setComment(String comment)
    {
        this.comment = comment;
        return this;
    }

    public abstract Collection< ? > list() throws JCRQueryException;

    public static final class CriterionEntry implements Serializable
    {

        private static final long serialVersionUID = 1L;

        private final Criterion criterion;

        private final Criteria criteria;

        private CriterionEntry(Criterion criterion, Criteria criteria)
        {
            this.criteria = criteria;
            this.criterion = criterion;
        }

        public Criterion getCriterion()
        {
            return criterion;
        }

        public Criteria getCriteria()
        {
            return criteria;
        }

        @Override
        public String toString()
        {
            return criterion.toString();
        }
    }

    public static final class OrderEntry implements Serializable
    {

        private static final long serialVersionUID = 1L;

        private final Order order;

        private final Criteria criteria;

        private OrderEntry(Order order, Criteria criteria)
        {
            this.criteria = criteria;
            this.order = order;
        }

        public Order getOrder()
        {
            return order;
        }

        public Criteria getCriteria()
        {
            return criteria;
        }

        @Override
        public String toString()
        {
            return order.toString();
        }
    }
}